@@ -9,13 +9,11 @@ import (
 	"strconv"
 	"strings"
 
-	ssv1alpha1 "github.com/bitnami-labs/sealed-secrets/pkg/apis/sealed-secrets/v1alpha1"
 	"github.com/mitchellh/go-homedir"
 	"github.com/openshift/odo/pkg/log"
 	"github.com/spf13/afero"
 	corev1 "k8s.io/api/core/v1"
 	v1rbac "k8s.io/api/rbac/v1"
-	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
 	"github.com/redhat-developer/kam/pkg/pipelines/argocd"
@@ -33,10 +31,10 @@ import (
 	"github.com/redhat-developer/kam/pkg/pipelines/routes"
 	"github.com/redhat-developer/kam/pkg/pipelines/scm"
 	"github.com/redhat-developer/kam/pkg/pipelines/secrets"
-	"github.com/redhat-developer/kam/pkg/pipelines/statustracker"
 	"github.com/redhat-developer/kam/pkg/pipelines/tasks"
 	"github.com/redhat-developer/kam/pkg/pipelines/triggers"
 	"github.com/redhat-developer/kam/pkg/pipelines/yaml"
+	pipelinev1 "github.com/tektoncd/pipeline/pkg/apis/pipeline/v1beta1"
 )
 
 const (
@@ -47,20 +45,20 @@ const (
 	rolesPath             = "02-rolebindings/pipeline-service-role.yaml"
 	rolebindingsPath      = "02-rolebindings/pipeline-service-rolebinding.yaml"
 	serviceAccountPath    = "02-rolebindings/pipeline-service-account.yaml"
-	secretsPath           = "03-secrets/gitops-webhook-secret.yaml"     //nolint:gosec
-	authTokenPath         = "03-secrets/git-host-access-token.yaml"     // nolint:gosec
-	basicAuthTokenPath    = "03-secrets/git-host-basic-auth-token.yaml" // nolint:gosec
-	dockerConfigPath      = "03-secrets/docker-config.yaml"
-	gitopsTasksPath       = "04-tasks/deploy-from-source-task.yaml"
-	ciPipelinesPath       = "05-pipelines/ci-dryrun-from-push-pipeline.yaml"
-	appCiPipelinesPath    = "05-pipelines/app-ci-pipeline.yaml"
-	pushTemplatePath      = "07-templates/ci-dryrun-from-push-template.yaml"
-	appCIPushTemplatePath = "07-templates/app-ci-build-from-push-template.yaml"
-	eventListenerPath     = "08-eventlisteners/cicd-event-listener.yaml"
-	routePath             = "09-routes/gitops-webhook-event-listener.yaml"
+	gitopsTasksPath       = "03-tasks/deploy-from-source-task.yaml"
+	commitStatusTaskPath  = "03-tasks/set-commit-status-task.yaml"
+	ciPipelinesPath       = "04-pipelines/ci-dryrun-from-push-pipeline.yaml"
+	appCiPipelinesPath    = "04-pipelines/app-ci-pipeline.yaml"
+	pushTemplatePath      = "06-templates/ci-dryrun-from-push-template.yaml"
+	appCIPushTemplatePath = "06-templates/app-ci-build-from-push-template.yaml"
+	eventListenerPath     = "07-eventlisteners/cicd-event-listener.yaml"
+	routePath             = "08-routes/gitops-webhook-event-listener.yaml"
 
 	dockerSecretName = "regcred"
 
+	authTokenSecretName = "git-host-access-token"
+	basicAuthTokenName  = "git-host-basic-auth-token"
+
 	saName              = "pipeline"
 	roleBindingName     = "pipelines-service-role-binding"
 	webhookSecretLength = 20
@@ -77,16 +75,15 @@ type BootstrapOptions struct {
 	GitOpsWebhookSecret      string // This is the secret for authenticating hooks from your GitOps repo.
 	Prefix                   string
 	DockerConfigJSONFilename string
-	ImageRepo                string               // This is where built images are pushed to.
-	OutputPath               string               // Where to write the bootstrapped files to?
-	SealedSecretsService     types.NamespacedName // SealedSecrets Services name
-	GitHostAccessToken       string               // The auth token to use to send commit-status notifications, and access private repositories.
-	Overwrite                bool                 // This allows to overwrite if there is an existing gitops repository
-	ServiceRepoURL           string               // This is the full URL to your GitHub repository for your app source.
-	SaveTokenKeyRing         bool                 // If true, the access-token will be saved in the keyring
-	ServiceWebhookSecret     string               // This is the secret for authenticating hooks from your app source.
-	PrivateRepoDriver        string               // Records the type of the GitOpsRepoURL driver if not a well-known host.
-	CommitStatusTracker      bool                 // If true, this is a "private repository", i.e. requires authentication to clone the repository.
+	ImageRepo                string // This is where built images are pushed to.
+	OutputPath               string // Where to write the bootstrapped files to?
+	GitHostAccessToken       string // The auth token to use to access repositories.
+	Overwrite                bool   // This allows to overwrite if there is an existing gitops repository
+	ServiceRepoURL           string // This is the full URL to your GitHub repository for your app source.
+	SaveTokenKeyRing         bool   // If true, the access-token will be saved in the keyring
+	ServiceWebhookSecret     string // This is the secret for authenticating hooks from your app source.
+	PrivateRepoDriver        string // Records the type of the GitOpsRepoURL driver if not a well-known host.
+	PushToGit                bool   // If true, gitops repository is pushed to remote git repository.
 }
 
 // PolicyRules to be bound to service account
@@ -107,11 +104,6 @@ var (
 			Resources: []string{"clusterrolebindings", "rolebindings"},
 			Verbs:     []string{"get", "create", "patch"},
 		},
-		{
-			APIGroups: []string{"bitnami.com"},
-			Resources: []string{"sealedsecrets"},
-			Verbs:     []string{"get", "patch", "create"},
-		},
 		{
 			APIGroups: []string{"apps"},
 			Resources: []string{"deployments"},
@@ -128,7 +120,7 @@ var (
 // Bootstrap is the entry-point from the CLI for bootstrapping the GitOps
 // configuration.
 func Bootstrap(o *BootstrapOptions, appFs afero.Fs) error {
-	err := checkPipelinesFileExists(appFs, o.OutputPath, o.Overwrite)
+	err := checkPipelinesFileExists(appFs, o.OutputPath, o.Overwrite, o.PushToGit)
 	if err != nil {
 		return err
 	}
@@ -137,7 +129,7 @@ func Bootstrap(o *BootstrapOptions, appFs afero.Fs) error {
 		return err
 	}
 
-	bootstrapped, err := bootstrapResources(o, appFs)
+	bootstrapped, otherResources, err := bootstrapResources(o, appFs)
 	if err != nil {
 		return fmt.Errorf("failed to bootstrap resources: %v", err)
 	}
@@ -154,6 +146,10 @@ func Bootstrap(o *BootstrapOptions, appFs afero.Fs) error {
 	if err != nil {
 		return fmt.Errorf("failed to write resources: %w", err)
 	}
+	_, err = yaml.WriteResources(appFs, filepath.Join(o.OutputPath, ".."), otherResources)
+	if err != nil {
+		return fmt.Errorf("failed to write resources: %w", err)
+	}
 
 	return nil
 }
@@ -176,15 +172,15 @@ func maybeMakeHookSecrets(o *BootstrapOptions) error {
 	return nil
 }
 
-func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, error) {
+func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, res.Resources, error) {
 	ns := namespaces.NamesWithPrefix(o.Prefix)
 	appRepo, err := scm.NewRepository(o.ServiceRepoURL)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	repoName, err := repoFromURL(appRepo.URL())
 	if err != nil {
-		return nil, fmt.Errorf("invalid app repo URL: %v", err)
+		return nil, nil, fmt.Errorf("invalid app repo URL: %v", err)
 	}
 	// No image repo was supplied so create the default OS internal image registry
 	if o.ImageRepo == "" {
@@ -192,7 +188,7 @@ func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, err
 	}
 	isInternalRegistry, imageRepo, err := imagerepo.ValidateImageRepo(o.ImageRepo)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	log.Success("Options used:")
@@ -202,31 +198,30 @@ func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, err
 	if !isInternalRegistry {
 		log.Progressf("  Path to config.json: %s", o.DockerConfigJSONFilename)
 	}
-	log.Progressf("  Commit status tracker: %s", strconv.FormatBool(o.CommitStatusTracker))
 	log.Progressf("  Output folder: %s", o.OutputPath)
 	log.Progressf("  Overwrite output folder: %s", strconv.FormatBool(o.Overwrite))
 	log.Progressf("")
 
 	gitOpsRepo, err := scm.NewRepository(o.GitOpsRepoURL)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
-	bootstrapped, err := createInitialFiles(
+	bootstrapped, otherResources, err := createInitialFiles(
 		appFs, gitOpsRepo, o)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	appName := repoToAppName(repoName)
 	serviceName := repoName
 	secretName := secrets.MakeServiceWebhookSecretName(ns["dev"], serviceName)
 	envs, configEnv, err := bootstrapEnvironments(appRepo, o.Prefix, secretName, ns)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	if o.PrivateRepoDriver != "" {
 		host, err := scm.HostnameFromURL(o.GitOpsRepoURL)
 		if err != nil {
-			return nil, fmt.Errorf("failed to get hostname from URL %q: %w", o.GitOpsRepoURL, err)
+			return nil, nil, fmt.Errorf("failed to get hostname from URL %q: %w", o.GitOpsRepoURL, err)
 		}
 		configEnv.Git = &config.GitConfig{Drivers: map[string]string{host: o.PrivateRepoDriver}}
 	}
@@ -234,48 +229,45 @@ func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, err
 
 	devEnv := m.GetEnvironment(ns["dev"])
 	if devEnv == nil {
-		return nil, errors.New("unable to bootstrap without dev environment")
+		return nil, nil, errors.New("unable to bootstrap without dev environment")
 	}
 
 	app := m.GetApplication(ns["dev"], appName)
 	if app == nil {
-		return nil, errors.New("unable to bootstrap without application")
+		return nil, nil, errors.New("unable to bootstrap without application")
 	}
 	svcFiles, err := bootstrapServiceDeployment(devEnv, app)
 	if err != nil {
-		return nil, fmt.Errorf("failed to create bootstrap service: %w", err)
+		return nil, nil, fmt.Errorf("failed to create bootstrap service: %w", err)
 	}
-	hookSecret, err := secrets.CreateSealedSecret(
-		meta.NamespacedName(ns["cicd"], secretName),
-		o.SealedSecretsService,
+	var opaqueSecret *corev1.Secret
+	opaqueSecret, err = secrets.CreateUnsealedSecret(meta.NamespacedName(ns["cicd"], secretName),
 		o.ServiceWebhookSecret,
 		eventlisteners.WebhookSecretKey)
 	if err != nil {
-		return nil, fmt.Errorf("failed to generate GitHub Webhook Secret: %v", err)
+		return nil, nil, fmt.Errorf("failed to create secret")
 	}
 
 	cfg := m.GetPipelinesConfig()
 	if cfg == nil {
-		return nil, errors.New("failed to find a pipeline configuration - unable to continue bootstrap")
+		return nil, nil, errors.New("failed to find a pipeline configuration - unable to continue bootstrap")
 	}
-	secretFilename := filepath.Join("03-secrets", secretName+".yaml")
-	secretsPath := filepath.Join(config.PathForPipelines(cfg), "base", secretFilename)
-	bootstrapped[secretsPath] = hookSecret
-
+	secretFilename := filepath.ToSlash(filepath.Join("secrets", secretName+".yaml"))
+	otherResources[secretFilename] = opaqueSecret
 	bindingName, imageRepoBindingFilename, svcImageBinding := createSvcImageBinding(cfg, devEnv, appName, serviceName, imageRepo, !isInternalRegistry)
 	bootstrapped = res.Merge(svcImageBinding, bootstrapped)
 
 	kustomizePath := filepath.Join(config.PathForPipelines(cfg), "base", "kustomization.yaml")
 	k, ok := bootstrapped[kustomizePath].(res.Kustomization)
 	if !ok {
-		return nil, fmt.Errorf("no kustomization for the %s environment found", kustomizePath)
+		return nil, nil, fmt.Errorf("no kustomization for the %s environment found", kustomizePath)
 	}
 	if isInternalRegistry {
 		filenames, resources, err := imagerepo.CreateInternalRegistryResources(
 			cfg, roles.CreateServiceAccount(meta.NamespacedName(cfg.Name, saName)),
 			imageRepo, o.GitOpsRepoURL)
 		if err != nil {
-			return nil, fmt.Errorf("failed to get resources for internal image repository: %v", err)
+			return nil, nil, fmt.Errorf("failed to get resources for internal image repository: %v", err)
 		}
 		bootstrapped = res.Merge(resources, bootstrapped)
 		k.AddResources(filenames...)
@@ -289,11 +281,11 @@ func bootstrapResources(o *BootstrapOptions, appFs afero.Fs) (res.Resources, err
 	}
 	bootstrapped[pipelinesFile] = m
 
-	k.AddResources(secretFilename, imageRepoBindingFilename)
+	k.AddResources(imageRepoBindingFilename)
 	bootstrapped[kustomizePath] = k
 
 	bootstrapped = res.Merge(svcFiles, bootstrapped)
-	return bootstrapped, nil
+	return bootstrapped, otherResources, nil
 }
 
 func bootstrapServiceDeployment(dev *config.Environment, app *config.Application) (res.Resources, error) {
@@ -432,24 +424,48 @@ func defaultPipelines(r scm.Repository) *config.Pipelines {
 }
 
 // Checks whether the pipelines.yaml is present in the output path specified.
-func checkPipelinesFileExists(appFs afero.Fs, outputPath string, overWrite bool) error {
-	exists, _ := ioutils.IsExisting(appFs, filepath.Join(outputPath, pipelinesFile))
-	if exists && !overWrite {
-		return fmt.Errorf("pipelines.yaml in output path already exists. If you want to replace your existing files, please rerun with --overwrite")
+func checkPipelinesFileExists(appFs afero.Fs, outputPath string, overWrite bool, pushToGit bool) error {
+
+	if overWrite {
+		return nil
+	}
+	checkList := []string{pipelinesFile}
+	if pushToGit {
+		checkList = append(checkList, ".git")
+	}
+
+	if err := errorIfFileExists(appFs, outputPath, checkList...); err != nil {
+		return err
+	}
+
+	secretsFolderExists, _ := ioutils.IsExisting(appFs, filepath.Join(outputPath, "..", "secrets"))
+	if secretsFolderExists {
+		return fmt.Errorf("the secrets folder located as a sibling of the output folder %s already exists. Rerun with --overwrite", outputPath)
+	}
+
+	return nil
+}
+
+func errorIfFileExists(appFs afero.Fs, outputPath string, files ...string) error {
+	for _, file := range files {
+		exists, _ := ioutils.IsExisting(appFs, filepath.Join(outputPath, file))
+		if exists {
+			return fmt.Errorf("%s in output path already exists. If you want to replace your existing files, please rerun with --overwrite", file)
+		}
 	}
 	return nil
 }
 
-func createInitialFiles(fs afero.Fs, repo scm.Repository, o *BootstrapOptions) (res.Resources, error) {
+func createInitialFiles(fs afero.Fs, repo scm.Repository, o *BootstrapOptions) (res.Resources, res.Resources, error) {
 	cicd := &config.PipelinesConfig{Name: o.Prefix + "cicd"}
 	pipelineConfig := &config.Config{Pipelines: cicd}
 	manifest := createManifest(repo.URL(), pipelineConfig)
 	initialFiles := res.Resources{
 		pipelinesFile: manifest,
 	}
-	resources, err := createCICDResources(fs, repo, cicd, o)
+	resources, otherResources, err := createCICDResources(fs, repo, cicd, o)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	files := getResourceFiles(resources)
@@ -461,11 +477,11 @@ func createInitialFiles(fs afero.Fs, repo scm.Repository, o *BootstrapOptions) (
 		getCICDKustomization(files))
 	initialFiles = res.Merge(pipelinesConfigKustomizations, initialFiles)
 
-	return initialFiles, nil
+	return initialFiles, otherResources, nil
 }
 
 // createDockerSecret creates a secret that allows pushing images to upstream repositories.
-func createDockerSecret(fs afero.Fs, dockerConfigJSONFilename, secretNS string, sealedSecretsService types.NamespacedName) (*ssv1alpha1.SealedSecret, error) {
+func createDockerSecret(fs afero.Fs, dockerConfigJSONFilename, secretNS string) (*corev1.Secret, error) {
 	if dockerConfigJSONFilename == "" {
 		return nil, errors.New("failed to generate path to file: --dockerconfigjson flag is not provided")
 	}
@@ -479,79 +495,79 @@ func createDockerSecret(fs afero.Fs, dockerConfigJSONFilename, secretNS string,
 	}
 	defer f.Close()
 
-	dockerSecret, err := secrets.CreateSealedDockerConfigSecret(meta.NamespacedName(secretNS, dockerSecretName), sealedSecretsService, f)
+	dockerSecret, err := secrets.CreateUnsealedDockerConfigSecret(meta.NamespacedName(secretNS, dockerSecretName), f)
 	if err != nil {
 		return nil, err
 	}
-
 	return dockerSecret, nil
 }
 
 // createCICDResources creates resources for OpenShift pipelines.
-func createCICDResources(fs afero.Fs, repo scm.Repository, pipelineConfig *config.PipelinesConfig, o *BootstrapOptions) (res.Resources, error) {
+func createCICDResources(fs afero.Fs, repo scm.Repository, pipelineConfig *config.PipelinesConfig, o *BootstrapOptions) (res.Resources, res.Resources, error) {
 	cicdNamespace := pipelineConfig.Name
 	// key: path of the resource
 	// value: YAML content of the resource
 	outputs := map[string]interface{}{}
-	githubSecret, err := secrets.CreateSealedSecret(meta.NamespacedName(cicdNamespace, eventlisteners.GitOpsWebhookSecret),
-		o.SealedSecretsService, o.GitOpsWebhookSecret, eventlisteners.WebhookSecretKey)
+	otherOutputs := map[string]interface{}{}
+	githubSecret, err := secrets.CreateUnsealedSecret(meta.NamespacedName(cicdNamespace, eventlisteners.GitOpsWebhookSecret), o.GitOpsWebhookSecret, eventlisteners.WebhookSecretKey)
 	if err != nil {
-		return nil, fmt.Errorf("failed to generate GitHub Webhook Secret: %w", err)
+		return nil, nil, fmt.Errorf("failed to generate GitHub Webhook Secret: %w", err)
 	}
-	outputs[secretsPath] = githubSecret
+	unEncSecretPath := filepath.Join("secrets", "gitops-webhook-secret.yaml")
+	otherOutputs[unEncSecretPath] = githubSecret
 	outputs[namespacesPath] = namespaces.Create(cicdNamespace, o.GitOpsRepoURL)
 	outputs[rolesPath] = roles.CreateClusterRole(meta.NamespacedName("", roles.ClusterRoleName), Rules)
 
 	sa := roles.CreateServiceAccount(meta.NamespacedName(cicdNamespace, saName))
 
 	if o.DockerConfigJSONFilename != "" {
-		dockerSecret, err := createDockerSecret(fs, o.DockerConfigJSONFilename, cicdNamespace,
-			o.SealedSecretsService)
+		dockerUnencryptedSecret, err := createDockerSecret(fs, o.DockerConfigJSONFilename, cicdNamespace)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
+		}
+		if dockerUnencryptedSecret != nil {
+			otherOutputs[filepath.Join("secrets", "docker-config.yaml")] = dockerUnencryptedSecret
+			log.Success("Authentication tokens for docker config not sealed in secrets")
 		}
-		outputs[dockerConfigPath] = dockerSecret
-		log.Success("Authentication tokens encrypted in secrets")
 		outputs[serviceAccountPath] = roles.AddSecretToSA(sa, dockerSecretName)
 	}
 
 	if o.GitHostAccessToken != "" {
-		err := generateSecrets(outputs, sa, cicdNamespace, o)
+		err := generateSecrets(outputs, otherOutputs, sa, cicdNamespace, o)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 	}
 
-	if o.CommitStatusTracker {
-		trackerResources, err := statustracker.Resources(cicdNamespace, o.GitOpsRepoURL, o.PrivateRepoDriver)
-		if err != nil {
-			return nil, err
-		}
-		outputs = res.Merge(outputs, trackerResources)
-		log.Success("Pipelines tracker has been configured")
-	}
-
 	outputs[rolebindingsPath] = roles.CreateClusterRoleBinding(meta.NamespacedName("", roleBindingName), sa, "ClusterRole", roles.ClusterRoleName)
 	script, err := dryrun.MakeScript("kubectl", cicdNamespace)
 	if err != nil {
-		return nil, err
+		return nil, otherOutputs, err
 	}
 	outputs[gitopsTasksPath] = tasks.CreateDeployFromSourceTask(cicdNamespace, script)
-	outputs[ciPipelinesPath] = pipelines.CreateCIPipeline(meta.NamespacedName(cicdNamespace, "ci-dryrun-from-push-pipeline"), cicdNamespace)
-	outputs[appCiPipelinesPath] = pipelines.CreateAppCIPipeline(meta.NamespacedName(cicdNamespace, "app-ci-pipeline"))
+	// currently, the commit status task doesn't support enterprise repository
+	// enable it by default once the status task supports enterprise repository
+	if o.PrivateRepoDriver == "" {
+		outputs[commitStatusTaskPath] = tasks.CreateCommitStatusTask(cicdNamespace)
+	}
+	outputs[ciPipelinesPath] = removeCommitStatus(pipelines.CreateCIPipeline(meta.NamespacedName(cicdNamespace, "ci-dryrun-from-push-pipeline"), cicdNamespace), o.PrivateRepoDriver)
+	outputs[appCiPipelinesPath] = removeCommitStatus(pipelines.CreateAppCIPipeline(meta.NamespacedName(cicdNamespace, "app-ci-pipeline")), o.PrivateRepoDriver)
 	pushBinding, pushBindingName := repo.CreatePushBinding(cicdNamespace)
-	outputs[filepath.Join("06-bindings", pushBindingName+".yaml")] = pushBinding
+	outputs[filepath.ToSlash(filepath.Join("05-bindings", pushBindingName+".yaml"))] = pushBinding
 	outputs[pushTemplatePath] = triggers.CreateCIDryRunTemplate(cicdNamespace, saName)
 	outputs[appCIPushTemplatePath] = triggers.CreateDevCIBuildPRTemplate(cicdNamespace, saName)
-	outputs[eventListenerPath] = eventlisteners.Generate(repo, cicdNamespace, saName, eventlisteners.GitOpsWebhookSecret)
+	outputs[eventListenerPath], err = eventlisteners.Generate(repo, cicdNamespace, saName, eventlisteners.GitOpsWebhookSecret)
+	if err != nil {
+		return nil, nil, err
+	}
 	log.Success("OpenShift Pipelines resources created")
 	route, err := eventlisteners.GenerateRoute(cicdNamespace)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	outputs[routePath] = route
 	log.Success("Openshift Route for EventListener created")
-	return outputs, nil
+	return outputs, otherOutputs, nil
 }
 
 func createManifest(gitOpsRepoURL string, configEnv *config.Config, envs ...*config.Environment) *config.Manifest {
@@ -595,28 +611,44 @@ func getResourceFiles(r res.Resources) []string {
 	return files
 }
 
-func generateSecrets(outputs res.Resources, sa *corev1.ServiceAccount, ns string, o *BootstrapOptions) error {
-	if o.CommitStatusTracker {
-		tokenSecret, err := secrets.CreateSealedSecret(meta.NamespacedName(
-			ns, statustracker.CommitStatusTrackerSecret), o.SealedSecretsService, o.GitHostAccessToken, "token")
-		if err != nil {
-			return fmt.Errorf("failed to generate access token Secret: %w", err)
-		}
-		outputs[authTokenPath] = tokenSecret
-		outputs[serviceAccountPath] = roles.AddSecretToSA(sa, tokenSecret.Name)
+func generateSecrets(outputs res.Resources, otherOutputs res.Resources, sa *corev1.ServiceAccount, ns string, o *BootstrapOptions) error {
+	tokenSecret, err := secrets.CreateUnsealedSecret(meta.NamespacedName(
+		ns, authTokenSecretName), o.GitHostAccessToken, "token")
+	if err != nil {
+		return fmt.Errorf("failed to generate Secret: %w", err)
 	}
+	otherOutputs[filepath.Join("secrets", "git-host-access-token.yaml")] = tokenSecret
+	outputs[serviceAccountPath] = roles.AddSecretToSA(sa, tokenSecret.Name)
+
+	// basic auth token is used by Tekton pipelines to access private repositories
 	secretTargetHost, err := repoURL(o.ServiceRepoURL)
 	if err != nil {
 		return fmt.Errorf("failed to parse the Service Repo URL %q: %w", o.ServiceRepoURL, err)
 	}
-	basicAuthSecret, err := secrets.CreateSealedBasicAuthSecret(meta.NamespacedName(
-		ns, "git-host-basic-auth-token"), o.SealedSecretsService, o.GitHostAccessToken, meta.AddAnnotations(map[string]string{
+	basicAuthSecret := secrets.CreateUnsealedBasicAuthSecret(meta.NamespacedName(
+		ns, basicAuthTokenName), o.GitHostAccessToken, meta.AddAnnotations(map[string]string{
 		"tekton.dev/git-0": secretTargetHost,
 	}))
-	if err != nil {
-		return fmt.Errorf("failed to generate basic auth token Secret: %w", err)
-	}
-	outputs[basicAuthTokenPath] = basicAuthSecret
+	otherOutputs[filepath.Join("secrets", basicAuthTokenName+".yaml")] = basicAuthSecret
 	outputs[serviceAccountPath] = roles.AddSecretToSA(sa, basicAuthSecret.Name)
 	return nil
 }
+
+// remove the commit status task and it's dependency
+func removeCommitStatus(pipeline *pipelinev1.Pipeline, driver string) *pipelinev1.Pipeline {
+	if driver == "" {
+		return pipeline
+	}
+	pipeline.Spec.Finally = nil
+	tasks := []pipelinev1.PipelineTask{}
+	for _, task := range pipeline.Spec.Tasks {
+		if len(task.RunAfter) > 0 && task.RunAfter[0] == pipelines.PendingCommitStatusTask {
+			task.RunAfter = nil
+		}
+		if task.Name != pipelines.PendingCommitStatusTask {
+			tasks = append(tasks, task)
+		}
+	}
+	pipeline.Spec.Tasks = tasks
+	return pipeline
+}
