@@ -45,5 +45,5 @@ func ParsePipelinesFolder(fs afero.Fs, folderPath string) (*Manifest, error) {
 	if !info.IsDir() {
 		return nil, fmt.Errorf("the path %q is a file path (required directory path)", folderPath)
 	}
-	return ParseFile(fs, filepath.Join(folderPath, PipelinesFile))
+	return ParseFile(fs, filepath.Join(folderPath, PipelinesFile)) // Don't call filepath.ToSlash
 }
