@@ -1,8 +1,6 @@
 package pipelines
 
 import (
-	"crypto/rand"
-	"crypto/rsa"
 	"fmt"
 	"path/filepath"
 	"testing"
@@ -16,27 +14,27 @@ import (
 	"github.com/redhat-developer/kam/pkg/pipelines/meta"
 	res "github.com/redhat-developer/kam/pkg/pipelines/resources"
 	"github.com/redhat-developer/kam/pkg/pipelines/secrets"
+	"github.com/redhat-developer/kam/pkg/pipelines/triggers"
 	"github.com/spf13/afero"
 	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1alpha1"
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/types"
 	"sigs.k8s.io/yaml"
 )
 
 func TestServiceResourcesWithCICD(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
 	fakeFs := ioutils.NewMemoryFilesystem()
 	m := buildManifest(true, false)
-	hookSecret, err := secrets.CreateSealedSecret(
+	hookSecret, err := secrets.CreateUnsealedSecret(
 		meta.NamespacedName(
 			"cicd", "webhook-secret-test-dev-test"),
-		meta.NamespacedName("test-ns", "service"),
 		"123",
 		eventlisteners.WebhookSecretKey)
 	assertNoError(t, err)
 
+	wantOther := res.Resources{
+		"secrets/webhook-secret-test-dev-test.yaml": hookSecret,
+	}
 	want := res.Resources{
-		"config/cicd/base/03-secrets/webhook-secret-test-dev-test.yaml": hookSecret,
 		"environments/test-dev/apps/test-app/base/kustomization.yaml": &res.Kustomization{
 			Bases: []string{"../services/test-svc", "../services/test"}},
 		"environments/test-dev/apps/test-app/kustomization.yaml": &res.Kustomization{
@@ -78,16 +76,22 @@ func TestServiceResourcesWithCICD(t *testing.T) {
 											Namespace: "cicd",
 										},
 									},
+									Pipelines: &config.Pipelines{
+										Integration: &config.TemplateBinding{Bindings: []string{"test-dev-test-app-test-binding", "github-push-binding"}},
+									},
 								},
 							},
 						},
 					},
+					Pipelines: &config.Pipelines{
+						Integration: &config.TemplateBinding{Template: "app-ci-template", Bindings: []string{"github-push-binding"}},
+					},
 				},
 			},
 		},
 	}
 
-	got, err := serviceResources(m, fakeFs, &AddServiceOptions{
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
 		AppName:             "test-app",
 		EnvName:             "test-dev",
 		GitRepoURL:          "http://github.com/org/test",
@@ -102,10 +106,15 @@ func TestServiceResourcesWithCICD(t *testing.T) {
 	})); diff != "" {
 		t.Fatalf("serviceResources() failed: %v", diff)
 	}
+	if diff := cmp.Diff(1, len(otherResources)); diff != "" {
+		t.Fatalf("other resources contain one entry:\n%s", diff)
+	}
+	if diff := cmp.Diff(otherResources, wantOther); diff != "" {
+		t.Fatalf("serviceResources() failed to create secrets properly:\n%s", diff)
+	}
 }
 
 func TestServiceResourcesWithArgoCD(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
 	fakeFs := ioutils.NewMemoryFilesystem()
 	m := buildManifest(false, true)
 
@@ -153,7 +162,7 @@ func TestServiceResourcesWithArgoCD(t *testing.T) {
 		},
 	}
 
-	got, err := serviceResources(m, fakeFs, &AddServiceOptions{
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
 		AppName:             "test-app",
 		EnvName:             "test-dev",
 		GitRepoURL:          "http://github.com/org/test",
@@ -168,6 +177,9 @@ func TestServiceResourcesWithArgoCD(t *testing.T) {
 	})); diff != "" {
 		t.Fatalf("serviceResources() failed: %v", diff)
 	}
+	if diff := cmp.Diff(0, len(otherResources)); diff != "" {
+		t.Fatalf("other resources is not empty:\n%s", diff)
+	}
 }
 
 func TestServiceResourcesWithoutArgoCD(t *testing.T) {
@@ -212,7 +224,7 @@ func TestServiceResourcesWithoutArgoCD(t *testing.T) {
 		},
 	}
 
-	got, err := serviceResources(m, fakeFs, &AddServiceOptions{
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
 		AppName:             "test-app",
 		EnvName:             "test-dev",
 		GitRepoURL:          "http://github.com/org/test",
@@ -227,6 +239,9 @@ func TestServiceResourcesWithoutArgoCD(t *testing.T) {
 	})); diff != "" {
 		t.Fatalf("serviceResources() failed: %v", diff)
 	}
+	if diff := cmp.Diff(0, len(otherResources)); diff != "" {
+		t.Fatalf("other resources is not empty:\n%s", diff)
+	}
 }
 
 func TestAddServiceWithoutApp(t *testing.T) {
@@ -270,7 +285,7 @@ func TestAddServiceWithoutApp(t *testing.T) {
 		},
 	}
 
-	got, err := serviceResources(m, fakeFs, &AddServiceOptions{
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
 		AppName:             "new-app",
 		EnvName:             "test-dev",
 		GitRepoURL:          "http://github.com/org/test",
@@ -284,14 +299,15 @@ func TestAddServiceWithoutApp(t *testing.T) {
 			t.Fatalf("serviceResources() failed: %v", diff)
 		}
 	}
+	if diff := cmp.Diff(0, len(otherResources)); diff != "" {
+		t.Fatalf("other resources is not empty:\n%s", diff)
+	}
 }
 
 func TestAddServiceFilePaths(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
-
 	fakeFs := ioutils.NewMemoryFilesystem()
 	outputPath := afero.GetTempDir(fakeFs, "test")
-	pipelinesPath := filepath.Join(outputPath, pipelinesFile)
+	pipelinesPath := filepath.Join(outputPath, pipelinesFile) // Don't call filepath.ToSlash
 	m := buildManifest(true, true)
 	b, err := yaml.Marshal(m)
 	assertNoError(t, err)
@@ -304,7 +320,6 @@ func TestAddServiceFilePaths(t *testing.T) {
 		"environments/test-dev/apps/new-app/services/test/base/kustomization.yaml",
 		"environments/test-dev/apps/new-app/services/test/overlays/kustomization.yaml",
 		"environments/test-dev/apps/new-app/services/test/kustomization.yaml",
-		"config/cicd/base/03-secrets/webhook-secret-test-dev-test.yaml",
 		"config/cicd/base/kustomization.yaml",
 		"pipelines.yaml",
 		"config/argocd/test-dev-test-app-app.yaml",
@@ -322,7 +337,7 @@ func TestAddServiceFilePaths(t *testing.T) {
 	for _, path := range wantedPaths {
 		t.Run(fmt.Sprintf("checking path %s already exists", path), func(rt *testing.T) {
 			// The inmemory version of Afero doesn't return errors
-			exists, _ := fakeFs.Exists(filepath.Join(outputPath, path))
+			exists, _ := fakeFs.Exists(filepath.Join(outputPath, path)) // Don't call filepath.ToSlash
 			if !exists {
 				t.Fatalf("The file is not present at : %v", path)
 			}
@@ -331,11 +346,9 @@ func TestAddServiceFilePaths(t *testing.T) {
 }
 
 func TestAddServiceFolderPaths(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
-
 	fakeFs := ioutils.NewMemoryFilesystem()
 	outputPath := afero.GetTempDir(fakeFs, "test")
-	pipelinesPath := filepath.Join(outputPath, pipelinesFile)
+	pipelinesPath := filepath.Join(outputPath, pipelinesFile) // Don't call filepath.ToSlash
 	m := buildManifest(true, true)
 	b, err := yaml.Marshal(m)
 	assertNoError(t, err)
@@ -356,7 +369,7 @@ func TestAddServiceFolderPaths(t *testing.T) {
 	for _, path := range wantedPaths {
 		t.Run(fmt.Sprintf("checking path %s already exists", path), func(rt *testing.T) {
 			// The inmemory version of Afero doesn't return errors
-			exists, _ := fakeFs.DirExists(filepath.Join(outputPath, path))
+			exists, _ := fakeFs.DirExists(filepath.Join(outputPath, path)) // Don't call filepath.ToSlash
 			if !exists {
 				t.Fatalf("The directory does not exist at path : %v", path)
 			}
@@ -365,7 +378,6 @@ func TestAddServiceFolderPaths(t *testing.T) {
 }
 
 func TestServiceWithArgoCD(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
 	fakeFs := ioutils.NewMemoryFilesystem()
 	m := buildManifest(true, true)
 	want := res.Resources{
@@ -405,10 +417,16 @@ func TestServiceWithArgoCD(t *testing.T) {
 											Namespace: "cicd",
 										},
 									},
+									Pipelines: &config.Pipelines{
+										Integration: &config.TemplateBinding{Bindings: []string{"test-dev-test-app-test-binding", "github-push-binding"}},
+									},
 								},
 							},
 						},
 					},
+					Pipelines: &config.Pipelines{
+						Integration: &config.TemplateBinding{Template: "app-ci-template", Bindings: []string{"github-push-binding"}},
+					},
 				},
 			},
 		},
@@ -416,7 +434,7 @@ func TestServiceWithArgoCD(t *testing.T) {
 	argo, err := argocd.Build(argocd.ArgoCDNamespace, "http://github.com/org/test", m)
 	assertNoError(t, err)
 	want = res.Merge(argo, want)
-	got, err := serviceResources(m, fakeFs, &AddServiceOptions{
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
 		AppName:             "test-app",
 		EnvName:             "test-dev",
 		GitRepoURL:          "http://github.com/org/test",
@@ -431,14 +449,15 @@ func TestServiceWithArgoCD(t *testing.T) {
 	})); diff != "" {
 		t.Fatalf("serviceResources() failed: %v", diff)
 	}
+	if diff := cmp.Diff(1, len(otherResources)); diff != "" {
+		t.Fatalf("other resources is not empty:\n%s", diff)
+	}
 }
 
 func TestAddServiceWithImageWithNoPipelines(t *testing.T) {
-	defer stubDefaultPublicKeyFunc(t)()
-
 	fakeFs := ioutils.NewMemoryFilesystem()
 	outputPath := afero.GetTempDir(fakeFs, "test")
-	pipelinesPath := filepath.Join(outputPath, pipelinesFile)
+	pipelinesPath := filepath.Join(outputPath, pipelinesFile) // Don't call filepath.ToSlash
 	m := buildManifest(true, true)
 	m.Environments = append(m.Environments, &config.Environment{
 		Name: "staging",
@@ -463,7 +482,7 @@ func TestAddServiceWithImageWithNoPipelines(t *testing.T) {
 	for _, path := range wantedPaths {
 		t.Run(fmt.Sprintf("checking path %s already exists", path), func(rt *testing.T) {
 			// The inmemory version of Afero doesn't return errors
-			exists, _ := fakeFs.DirExists(filepath.Join(outputPath, path))
+			exists, _ := fakeFs.DirExists(filepath.Join(outputPath, path)) // Don't call filepath.ToSlash
 			if !exists {
 				t.Fatalf("The directory does not exist at path : %v", path)
 			}
@@ -471,6 +490,113 @@ func TestAddServiceWithImageWithNoPipelines(t *testing.T) {
 	}
 }
 
+func TestAddServiceWithoutImage(t *testing.T) {
+	fakeFs := ioutils.NewMemoryFilesystem()
+	outputPath := afero.GetTempDir(fakeFs, "test")
+	pipelinesPath := filepath.ToSlash(filepath.Join(outputPath, pipelinesFile))
+	m := buildManifest(true, true)
+	m.Environments = append(m.Environments, &config.Environment{
+		Name: "staging",
+	})
+	b, err := yaml.Marshal(m)
+	assertNoError(t, err)
+	err = afero.WriteFile(fakeFs, pipelinesPath, b, 0644)
+	assertNoError(t, err)
+
+	err = AddService(&AddServiceOptions{
+		AppName:             "new-app",
+		EnvName:             "staging",
+		GitRepoURL:          "http://github.com/org/test",
+		PipelinesFolderPath: outputPath,
+		WebhookSecret:       "123",
+		ServiceName:         "test",
+	}, fakeFs)
+	assertNoError(t, err)
+
+	files := res.Resources{
+		"config/cicd/base/05-bindings/staging-new-app-test-binding.yaml": triggers.CreateImageRepoBinding("cicd", "staging-new-app-test-binding", "image-registry.openshift-image-registry.svc:5000/cicd/test", "false"),
+	}
+
+	for path, resource := range files {
+		t.Run(fmt.Sprintf("checking path %s already exists", path), func(rt *testing.T) {
+			filePath := filepath.Join(outputPath, path) // Don't call filepath.ToSlash
+			exists, err := fakeFs.Exists(filePath)
+			assertNoError(t, err)
+			if !exists {
+				t.Fatalf("The file does not exist at path : %s", filepath.Join(outputPath, path)) // Don't call filepath.ToSlash
+			}
+			got, err := fakeFs.ReadFile(filePath)
+			assertNoError(t, err)
+
+			want, err := yaml.Marshal(resource)
+			assertNoError(t, err)
+
+			if diff := cmp.Diff(want, got); diff != "" {
+				t.Fatalf("AddServices failed: %s", diff)
+			}
+		})
+	}
+}
+
+func TestAddServiceWithoutGitRepoUrl(t *testing.T) {
+	fakeFs := ioutils.NewMemoryFilesystem()
+	m := buildManifest(false, false)
+	want := res.Resources{
+		"environments/test-dev/apps/new-app/base/kustomization.yaml":     &res.Kustomization{Bases: []string{"../services/test"}},
+		"environments/test-dev/apps/new-app/overlays/kustomization.yaml": &res.Kustomization{Bases: []string{"../base"}},
+		"environments/test-dev/apps/new-app/kustomization.yaml": &res.Kustomization{
+			Bases:        []string{"overlays"},
+			CommonLabels: map[string]string{"app.openshift.io/vcs-source": "org/test"},
+		},
+		"environments/test-dev/apps/new-app/services/test/base/kustomization.yaml":          &res.Kustomization{Bases: []string{"./config"}},
+		"environments/test-dev/apps/new-app/services/test/kustomization.yaml":               &res.Kustomization{Bases: []string{"overlays"}},
+		"environments/test-dev/apps/new-app/services/test/overlays/kustomization.yaml":      &res.Kustomization{Bases: []string{"../base"}},
+		"environments/cicd/base/pipelines/03-secrets/webhook-secret-test-dev-test-svc.yaml": nil,
+		"pipelines.yaml": &config.Manifest{
+			GitOpsURL: "http://github.com/org/test",
+			Environments: []*config.Environment{
+				{
+					Name: "test-dev",
+					Apps: []*config.Application{
+						{
+							Name: "test-app",
+							Services: []*config.Service{
+								{
+									Name:      "test-svc",
+									SourceURL: "https://github.com/myproject/test-svc",
+								},
+							},
+						},
+						{
+							Name: "new-app",
+							Services: []*config.Service{
+								{Name: "test"},
+							},
+						},
+					},
+				},
+			},
+		},
+	}
+
+	got, otherResources, err := serviceResources(m, fakeFs, &AddServiceOptions{
+		AppName:             "new-app",
+		EnvName:             "test-dev",
+		PipelinesFolderPath: pipelinesFile,
+		WebhookSecret:       "123",
+		ServiceName:         "test",
+	})
+	assertNoError(t, err)
+	for i := range want {
+		if diff := cmp.Diff(want[i], got[i]); diff != "" {
+			t.Fatalf("serviceResources() failed: %v", diff)
+		}
+	}
+	if diff := cmp.Diff(0, len(otherResources)); diff != "" {
+		t.Fatalf("other resources is not empty:\n%s", diff)
+	}
+}
+
 func buildManifest(withPipelines, withArgoCD bool) *config.Manifest {
 	m := config.Manifest{
 		GitOpsURL: "http://github.com/org/test",
@@ -536,7 +662,7 @@ func TestCreateSvcImageBinding(t *testing.T) {
 	if diff := cmp.Diff(bindingName, "new-env-newapp-new-svc-binding"); diff != "" {
 		t.Errorf("bindingName failed: %v", diff)
 	}
-	if diff := cmp.Diff(bindingFilename, "06-bindings/new-env-newapp-new-svc-binding.yaml"); diff != "" {
+	if diff := cmp.Diff(bindingFilename, "05-bindings/new-env-newapp-new-svc-binding.yaml"); diff != "" {
 		t.Errorf("bindingFilename failed: %v", diff)
 	}
 
@@ -557,23 +683,8 @@ func TestCreateSvcImageBinding(t *testing.T) {
 		},
 	}
 
-	wantResources := res.Resources{"config/cicd/base/06-bindings/new-env-newapp-new-svc-binding.yaml": triggerBinding}
+	wantResources := res.Resources{"config/cicd/base/05-bindings/new-env-newapp-new-svc-binding.yaml": triggerBinding}
 	if diff := cmp.Diff(resources, wantResources); diff != "" {
 		t.Errorf("resources failed: %v", diff)
 	}
 }
-
-func stubDefaultPublicKeyFunc(t *testing.T) func() {
-	origDefaultPublicKeyFunc := secrets.DefaultPublicKeyFunc
-	secrets.DefaultPublicKeyFunc = func(types.NamespacedName) (*rsa.PublicKey, error) {
-		key, err := rsa.GenerateKey(rand.Reader, 2048)
-		if err != nil {
-			t.Fatalf("failed to generate a private RSA key: %s", err)
-		}
-		return &key.PublicKey, nil
-	}
-
-	return func() {
-		secrets.DefaultPublicKeyFunc = origDefaultPublicKeyFunc
-	}
-}
