@@ -15,22 +15,30 @@
 package interpreter
 
 import (
+	"reflect"
 	"testing"
 
-	"github.com/golang/protobuf/ptypes"
-
-	"github.com/google/cel-go/common/packages"
+	"github.com/google/cel-go/checker"
+	"github.com/google/cel-go/checker/decls"
+	"github.com/google/cel-go/common"
+	"github.com/google/cel-go/common/containers"
 	"github.com/google/cel-go/common/types"
+	"github.com/google/cel-go/common/types/ref"
+	"github.com/google/cel-go/parser"
 
 	proto3pb "github.com/google/cel-go/test/proto3pb"
 
 	exprpb "google.golang.org/genproto/googleapis/api/expr/v1alpha1"
+	"google.golang.org/protobuf/types/known/anypb"
 )
 
-func TestAttributes_AbsoluteAttr(t *testing.T) {
-	reg := types.NewRegistry()
-	pkg := packages.NewPackage("acme.ns")
-	attrs := NewAttributeFactory(pkg, reg, reg)
+func TestAttributesAbsoluteAttr(t *testing.T) {
+	reg := newTestRegistry(t)
+	cont, err := containers.NewContainer(containers.Name("acme.ns"))
+	if err != nil {
+		t.Fatal(err)
+	}
+	attrs := NewAttributeFactory(cont, reg, reg)
 	vars, _ := NewActivation(map[string]interface{}{
 		"acme.a": map[string]interface{}{
 			"b": map[uint]interface{}{
@@ -56,11 +64,15 @@ func TestAttributes_AbsoluteAttr(t *testing.T) {
 	if out != types.String("success") {
 		t.Errorf("Got %v (%T), wanted success", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_AbsoluteAttr_Type(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesAbsoluteAttr_Type(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 
 	// int
 	attr := attrs.AbsoluteAttribute(1, "int")
@@ -71,11 +83,15 @@ func TestAttributes_AbsoluteAttr_Type(t *testing.T) {
 	if out != types.IntType {
 		t.Errorf("Got %v, wanted success", out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_RelativeAttr(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesRelativeAttr(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: []int32{2, 42},
@@ -91,10 +107,7 @@ func TestAttributes_RelativeAttr(t *testing.T) {
 	// }
 	//
 	// The expression being evaluated is: <map-literal>.a[-1][b] -> 42
-	op := &evalConst{
-		id:  1,
-		val: reg.NativeToValue(data),
-	}
+	op := NewConstValue(1, reg.NativeToValue(data))
 	attr := attrs.RelativeAttribute(1, op)
 	qualA, _ := attrs.NewQualifier(nil, 2, "a")
 	qualNeg1, _ := attrs.NewQualifier(nil, 3, int64(-1))
@@ -108,12 +121,19 @@ func TestAttributes_RelativeAttr(t *testing.T) {
 	if out != types.Int(42) {
 		t.Errorf("Got %v (%T), wanted 42", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_RelativeAttr_OneOf(t *testing.T) {
-	reg := types.NewRegistry()
-	pkg := packages.NewPackage("acme.ns")
-	attrs := NewAttributeFactory(pkg, reg, reg)
+func TestAttributesRelativeAttr_OneOf(t *testing.T) {
+	reg := newTestRegistry(t)
+	cont, err := containers.NewContainer(containers.Name("acme.ns"))
+	if err != nil {
+		t.Fatal(err)
+	}
+	attrs := NewAttributeFactory(cont, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: []int32{2, 42},
@@ -136,10 +156,7 @@ func TestAttributes_RelativeAttr_OneOf(t *testing.T) {
 	// - <map-literal>.a[-1][acme.b]
 	//
 	// The correct behavior should yield the value of the last alternative.
-	op := &evalConst{
-		id:  1,
-		val: reg.NativeToValue(data),
-	}
+	op := NewConstValue(1, reg.NativeToValue(data))
 	attr := attrs.RelativeAttribute(1, op)
 	qualA, _ := attrs.NewQualifier(nil, 2, "a")
 	qualNeg1, _ := attrs.NewQualifier(nil, 3, int64(-1))
@@ -153,11 +170,15 @@ func TestAttributes_RelativeAttr_OneOf(t *testing.T) {
 	if out != types.Int(42) {
 		t.Errorf("Got %v (%T), wanted 42", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_RelativeAttr_Conditional(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesRelativeAttr_Conditional(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: []int32{2, 42},
@@ -178,20 +199,14 @@ func TestAttributes_RelativeAttr_Conditional(t *testing.T) {
 	// <map-literal>.a[-1][(false ? b : c)[0]] -> 42
 	//
 	// Effectively the same as saying <map-literal>.a[-1][c[0]]
-	cond := &evalConst{
-		id:  2,
-		val: types.False,
-	}
+	cond := NewConstValue(2, types.False)
 	condAttr := attrs.ConditionalAttribute(4, cond,
 		attrs.AbsoluteAttribute(5, "b"),
 		attrs.AbsoluteAttribute(6, "c"))
 	qual0, _ := attrs.NewQualifier(nil, 7, 0)
 	condAttr.AddQualifier(qual0)
 
-	obj := &evalConst{
-		id:  1,
-		val: reg.NativeToValue(data),
-	}
+	obj := NewConstValue(1, reg.NativeToValue(data))
 	attr := attrs.RelativeAttribute(1, obj)
 	qualA, _ := attrs.NewQualifier(nil, 2, "a")
 	qualNeg1, _ := attrs.NewQualifier(nil, 3, int64(-1))
@@ -205,12 +220,19 @@ func TestAttributes_RelativeAttr_Conditional(t *testing.T) {
 	if out != types.Int(42) {
 		t.Errorf("Got %v (%T), wanted 42", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_RelativeAttr_Relative(t *testing.T) {
-	pkg := packages.NewPackage("acme.ns")
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(pkg, reg, reg)
+func TestAttributesRelativeAttr_Relative(t *testing.T) {
+	cont, err := containers.NewContainer(containers.Name("acme.ns"))
+	if err != nil {
+		t.Fatal(err)
+	}
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(cont, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: map[string]interface{}{
@@ -249,18 +271,12 @@ func TestAttributes_RelativeAttr_Relative(t *testing.T) {
 	//
 	// This is equivalent to:
 	//   <obj>.a[-1]["second"] -> 2u
-	obj := &evalConst{
-		id:  1,
-		val: reg.NativeToValue(data),
-	}
-	mp := &evalConst{
-		id: 1,
-		val: reg.NativeToValue(map[uint32]interface{}{
-			1: "first",
-			2: "second",
-			3: "third",
-		}),
-	}
+	obj := NewConstValue(1, reg.NativeToValue(data))
+	mp := NewConstValue(1, reg.NativeToValue(map[uint32]interface{}{
+		1: "first",
+		2: "second",
+		3: "third",
+	}))
 	relAttr := attrs.RelativeAttribute(4, mp)
 	qualB, _ := attrs.NewQualifier(nil, 5, attrs.AbsoluteAttribute(5, "b"))
 	relAttr.AddQualifier(qualB)
@@ -278,12 +294,19 @@ func TestAttributes_RelativeAttr_Relative(t *testing.T) {
 	if out != types.Uint(2) {
 		t.Errorf("Got %v (%T), wanted 2", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_OneofAttr(t *testing.T) {
-	reg := types.NewRegistry()
-	pkg := packages.NewPackage("acme.ns")
-	attrs := NewAttributeFactory(pkg, reg, reg)
+func TestAttributesOneofAttr(t *testing.T) {
+	reg := newTestRegistry(t)
+	cont, err := containers.NewContainer(containers.Name("acme.ns"))
+	if err != nil {
+		t.Fatal(err)
+	}
+	attrs := NewAttributeFactory(cont, reg, reg)
 	data := map[string]interface{}{
 		"a": map[string]interface{}{
 			"b": []int32{2, 42},
@@ -304,11 +327,15 @@ func TestAttributes_OneofAttr(t *testing.T) {
 	if out != "found" {
 		t.Errorf("Got %v, wanted 'found'", out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_ConditionalAttr_TrueBranch(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesConditionalAttr_TrueBranch(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: []int32{2, 42},
@@ -326,7 +353,7 @@ func TestAttributes_ConditionalAttr_TrueBranch(t *testing.T) {
 	fv := attrs.MaybeAttribute(3, "b")
 	qualC, _ := attrs.NewQualifier(nil, 4, "c")
 	fv.AddQualifier(qualC)
-	cond := attrs.ConditionalAttribute(1, &evalConst{val: types.True}, tv, fv)
+	cond := attrs.ConditionalAttribute(1, NewConstValue(0, types.True), tv, fv)
 	qualNeg1, _ := attrs.NewQualifier(nil, 5, int64(-1))
 	qual1, _ := attrs.NewQualifier(nil, 6, int64(1))
 	cond.AddQualifier(qualNeg1)
@@ -338,11 +365,15 @@ func TestAttributes_ConditionalAttr_TrueBranch(t *testing.T) {
 	if out != int32(42) {
 		t.Errorf("Got %v (%T), wanted 42", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(fv); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_ConditionalAttr_FalseBranch(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesConditionalAttr_FalseBranch(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 	data := map[string]interface{}{
 		"a": map[int]interface{}{
 			-1: []int32{2, 42},
@@ -360,7 +391,7 @@ func TestAttributes_ConditionalAttr_FalseBranch(t *testing.T) {
 	fv := attrs.MaybeAttribute(3, "b")
 	qualC, _ := attrs.NewQualifier(nil, 4, "c")
 	fv.AddQualifier(qualC)
-	cond := attrs.ConditionalAttribute(1, &evalConst{val: types.False}, tv, fv)
+	cond := attrs.ConditionalAttribute(1, NewConstValue(0, types.False), tv, fv)
 	qualNeg1, _ := attrs.NewQualifier(nil, 5, int64(-1))
 	qual1, _ := attrs.NewQualifier(nil, 6, int64(1))
 	cond.AddQualifier(qualNeg1)
@@ -372,23 +403,31 @@ func TestAttributes_ConditionalAttr_FalseBranch(t *testing.T) {
 	if out != uint(42) {
 		t.Errorf("Got %v (%T), wanted 42", out, out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(fv); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_ConditionalAttr_ErrorUnknown(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
+func TestAttributesConditionalAttr_ErrorUnknown(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
 
 	// err ? a : b
 	tv := attrs.AbsoluteAttribute(2, "a")
 	fv := attrs.MaybeAttribute(3, "b")
-	cond := attrs.ConditionalAttribute(1, &evalConst{val: types.NewErr("test error")}, tv, fv)
+	cond := attrs.ConditionalAttribute(1, NewConstValue(0, types.NewErr("test error")), tv, fv)
 	out, err := cond.Resolve(EmptyActivation())
 	if err == nil {
 		t.Errorf("Got %v, wanted error", out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(fv); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 
 	// unk ? a : b
-	condUnk := attrs.ConditionalAttribute(1, &evalConst{val: types.Unknown{1}}, tv, fv)
+	condUnk := attrs.ConditionalAttribute(1, NewConstValue(0, types.Unknown{1}), tv, fv)
 	out, err = condUnk.Resolve(EmptyActivation())
 	if err != nil {
 		t.Fatal(err)
@@ -397,12 +436,49 @@ func TestAttributes_ConditionalAttr_ErrorUnknown(t *testing.T) {
 	if !ok || !types.IsUnknown(unk) {
 		t.Errorf("Got %v, wanted unknown", out)
 	}
+	wantedMin, wantedMax = int64(1), int64(1)
+	if min, max := estimateCost(fv); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestResolver_CustomQualifier(t *testing.T) {
-	reg := types.NewRegistry()
+func BenchmarkResolverFieldQualifier(b *testing.B) {
+	msg := &proto3pb.TestAllTypes{
+		NestedType: &proto3pb.TestAllTypes_SingleNestedMessage{
+			SingleNestedMessage: &proto3pb.TestAllTypes_NestedMessage{
+				Bb: 123,
+			},
+		},
+	}
+	reg := newBenchRegistry(b, msg)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
+	vars, _ := NewActivation(map[string]interface{}{
+		"msg": msg,
+	})
+	attr := attrs.AbsoluteAttribute(1, "msg")
+	opType, found := reg.FindType("google.expr.proto3.test.TestAllTypes")
+	if !found {
+		b.Fatal("FindType() could not find TestAllTypes")
+	}
+	fieldType, found := reg.FindType("google.expr.proto3.test.TestAllTypes.NestedMessage")
+	if !found {
+		b.Fatal("FindType() could not find NestedMessage")
+	}
+	attr.AddQualifier(makeQualifier(b, attrs, opType.GetType(), 2, "single_nested_message"))
+	attr.AddQualifier(makeQualifier(b, attrs, fieldType.GetType(), 3, "bb"))
+	b.ResetTimer()
+	for i := 0; i < b.N; i++ {
+		_, err := attr.Resolve(vars)
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
+
+func TestResolverCustomQualifier(t *testing.T) {
+	reg := newTestRegistry(t)
 	attrs := &custAttrFactory{
-		AttributeFactory: NewAttributeFactory(packages.DefaultPackage, reg, reg),
+		AttributeFactory: NewAttributeFactory(containers.DefaultContainer, reg, reg),
 	}
 	msg := &proto3pb.TestAllTypes_NestedMessage{
 		Bb: 123,
@@ -424,12 +500,16 @@ func TestResolver_CustomQualifier(t *testing.T) {
 	if out != int32(123) {
 		t.Errorf("Got %v, wanted 123", out)
 	}
+	wantedMin, wantedMax := int64(1), int64(1)
+	if min, max := estimateCost(attr); min != wantedMin || max != wantedMax {
+		t.Errorf("Got cost interval [%v, %v], wanted [%v, %v]", min, max, wantedMin, wantedMax)
+	}
 }
 
-func TestAttributes_MissingMsg(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewAttributeFactory(packages.DefaultPackage, reg, reg)
-	any, _ := ptypes.MarshalAny(&proto3pb.TestAllTypes{})
+func TestAttributesMissingMsg(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewAttributeFactory(containers.DefaultContainer, reg, reg)
+	any, _ := anypb.New(&proto3pb.TestAllTypes{})
 	vars, _ := NewActivation(map[string]interface{}{
 		"missing_msg": any,
 	})
@@ -447,10 +527,10 @@ func TestAttributes_MissingMsg(t *testing.T) {
 	}
 }
 
-func TestAttributes_MissingMsg_UnknownField(t *testing.T) {
-	reg := types.NewRegistry()
-	attrs := NewPartialAttributeFactory(packages.DefaultPackage, reg, reg)
-	any, _ := ptypes.MarshalAny(&proto3pb.TestAllTypes{})
+func TestAttributeMissingMsg_UnknownField(t *testing.T) {
+	reg := newTestRegistry(t)
+	attrs := NewPartialAttributeFactory(containers.DefaultContainer, reg, reg)
+	any, _ := anypb.New(&proto3pb.TestAllTypes{})
 	vars, _ := NewPartialActivation(map[string]interface{}{
 		"missing_msg": any,
 	}, NewAttributePattern("missing_msg").QualString("field"))
@@ -469,10 +549,173 @@ func TestAttributes_MissingMsg_UnknownField(t *testing.T) {
 	}
 }
 
-func BenchmarkResolver_CustomQualifier(b *testing.B) {
-	reg := types.NewRegistry()
+func TestAttributeStateTracking(t *testing.T) {
+	var tests = []struct {
+		expr  string
+		env   []*exprpb.Decl
+		in    map[string]interface{}
+		out   ref.Val
+		state map[int64]interface{}
+	}{
+		{
+			expr: `[{"field": true}][0].field`,
+			env:  []*exprpb.Decl{},
+			in:   map[string]interface{}{},
+			out:  types.True,
+			state: map[int64]interface{}{
+				// overall expression
+				1: true,
+				// [{"field": true}][0]
+				6: map[ref.Val]ref.Val{types.String("field"): types.True},
+				// [{"field": true}][0].field
+				8: true,
+			},
+		},
+		{
+			expr: `a[1]['two']`,
+			env: []*exprpb.Decl{
+				decls.NewVar("a", decls.NewMapType(
+					decls.Int,
+					decls.NewMapType(decls.String, decls.Bool))),
+			},
+			in: map[string]interface{}{
+				"a": map[int64]interface{}{
+					1: map[string]bool{
+						"two": true,
+					},
+				},
+			},
+			out: types.True,
+			state: map[int64]interface{}{
+				// overall expression
+				1: true,
+				// a[1]
+				2: map[string]bool{"two": true},
+				// a[1]["two"]
+				4: true,
+			},
+		},
+		{
+			expr: `a[1][2][3]`,
+			env: []*exprpb.Decl{
+				decls.NewVar("a", decls.NewMapType(
+					decls.Int,
+					decls.NewMapType(decls.Dyn, decls.Dyn))),
+			},
+			in: map[string]interface{}{
+				"a": map[int64]interface{}{
+					1: map[int64]interface{}{
+						1: 0,
+						2: []string{"index", "middex", "outdex", "dex"},
+					},
+				},
+			},
+			out: types.String("dex"),
+			state: map[int64]interface{}{
+				// overall expression
+				1: "dex",
+				// a[1]
+				2: map[int64]interface{}{
+					1: 0,
+					2: []string{"index", "middex", "outdex", "dex"},
+				},
+				// a[1][2]
+				4: []string{"index", "middex", "outdex", "dex"},
+				// a[1][2][3]
+				6: "dex",
+			},
+		},
+		{
+			expr: `a[1][2][a[1][1]]`,
+			env: []*exprpb.Decl{
+				decls.NewVar("a", decls.NewMapType(
+					decls.Int,
+					decls.NewMapType(decls.Dyn, decls.Dyn))),
+			},
+			in: map[string]interface{}{
+				"a": map[int64]interface{}{
+					1: map[int64]interface{}{
+						1: 0,
+						2: []string{"index", "middex", "outdex", "dex"},
+					},
+				},
+			},
+			out: types.String("index"),
+			state: map[int64]interface{}{
+				// overall expression
+				1: "index",
+				// a[1]
+				2: map[int64]interface{}{
+					1: 0,
+					2: []string{"index", "middex", "outdex", "dex"},
+				},
+				// a[1][2]
+				4: []string{"index", "middex", "outdex", "dex"},
+				// a[1][2][a[1][1]]
+				6: "index",
+				// dynamic index into a[1][2]
+				// a[1]
+				8: map[int64]interface{}{
+					1: 0,
+					2: []string{"index", "middex", "outdex", "dex"},
+				},
+				// a[1][1]
+				10: int64(0),
+			},
+		},
+	}
+	for _, test := range tests {
+		tc := test
+		t.Run(tc.expr, func(tt *testing.T) {
+			src := common.NewTextSource(tc.expr)
+			parsed, errors := parser.Parse(src)
+			if len(errors.GetErrors()) != 0 {
+				tt.Fatalf(errors.ToDisplayString())
+			}
+			cont := containers.DefaultContainer
+			reg := newTestRegistry(t)
+			env := checker.NewStandardEnv(cont, reg)
+			if tc.env != nil {
+				env.Add(tc.env...)
+			}
+			checked, errors := checker.Check(parsed, src, env)
+			if len(errors.GetErrors()) != 0 {
+				tt.Fatalf(errors.ToDisplayString())
+			}
+			attrs := NewAttributeFactory(cont, reg, reg)
+			interp := NewStandardInterpreter(cont, reg, reg, attrs)
+			// Show that program planning will now produce an error.
+			st := NewEvalState()
+			i, err := interp.NewInterpretable(checked, Optimize(), TrackState(st))
+			if err != nil {
+				tt.Fatal(err)
+			}
+			if err != nil {
+				tt.Fatal(err)
+			}
+			in, _ := NewActivation(tc.in)
+			out := i.Eval(in)
+			if tc.out.Equal(out) != types.True {
+				tt.Errorf("got %v, wanted %v", out.Value(), tc.out)
+			}
+			for id, val := range tc.state {
+				stVal, found := st.Value(id)
+				if !found {
+					tt.Errorf("state not found for %d=%v", id, val)
+					continue
+				}
+				if !reflect.DeepEqual(stVal.Value(), val) {
+					tt.Errorf("got %v, wanted %v for id: %d", stVal.Value(), val, id)
+				}
+			}
+		})
+	}
+}
+
+func BenchmarkResolverCustomQualifier(b *testing.B) {
+	reg := newBenchRegistry(b)
 	attrs := &custAttrFactory{
-		AttributeFactory: NewAttributeFactory(packages.DefaultPackage, reg, reg),
+		AttributeFactory: NewAttributeFactory(containers.DefaultContainer, reg, reg),
 	}
 	msg := &proto3pb.TestAllTypes_NestedMessage{
 		Bb: 123,
@@ -517,3 +760,17 @@ func (q *nestedMsgQualifier) Qualify(vars Activation, obj interface{}) (interfac
 	pb := obj.(*proto3pb.TestAllTypes_NestedMessage)
 	return pb.GetBb(), nil
 }
+
+// Cost implements the Coster interface method. It returns zero for testing purposes.
+func (q *nestedMsgQualifier) Cost() (min, max int64) {
+	return 0, 0
+}
+
+func makeQualifier(b *testing.B, attrs AttributeFactory, typ *exprpb.Type, qualID int64, val interface{}) Qualifier {
+	b.Helper()
+	qual, err := attrs.NewQualifier(typ, qualID, val)
+	if err != nil {
+		b.Fatalf("attrs.NewQualifier() failed: %v", err)
+	}
+	return qual
+}
