@@ -6,7 +6,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"math/rand"
-	"reflect"
 	"strconv"
 	"strings"
 	"testing"
@@ -314,10 +313,26 @@ func TestTypes(t *testing.T) {
 	assert(t, (Result{Type: JSON}).Type.String() == "JSON")
 	assert(t, (Result{Type: 100}).Type.String() == "")
 	// bool
-	assert(t, (Result{Type: String, Str: "true"}).Bool())
-	assert(t, (Result{Type: True}).Bool())
+	assert(t, (Result{Type: True}).Bool() == true)
 	assert(t, (Result{Type: False}).Bool() == false)
-	assert(t, (Result{Type: Number, Num: 1}).Bool())
+	assert(t, (Result{Type: Number, Num: 1}).Bool() == true)
+	assert(t, (Result{Type: Number, Num: 0}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "1"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "T"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "t"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "true"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "True"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "TRUE"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "tRuE"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "0"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "f"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "F"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "false"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "False"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "FALSE"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "fAlSe"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "random"}).Bool() == false)
+
 	// int
 	assert(t, (Result{Type: String, Str: "1"}).Int() == 1)
 	assert(t, (Result{Type: True}).Int() == 1)
@@ -996,26 +1011,6 @@ var complicatedJSON = `
 }
 `
 
-func TestUnmarshal(t *testing.T) {
-	var s1 ComplicatedType
-	var s2 ComplicatedType
-	if err := json.Unmarshal([]byte(complicatedJSON), &s1); err != nil {
-		t.Fatal(err)
-	}
-	if err := Unmarshal([]byte(complicatedJSON), &s2); err != nil {
-		t.Fatal(err)
-	}
-	if !reflect.DeepEqual(&s1, &s2) {
-		t.Fatal("not equal")
-	}
-	var str string
-	err := json.Unmarshal([]byte(Get(complicatedJSON, "LeftOut").Raw), &str)
-	if err != nil {
-		t.Fatal(err)
-	}
-	assert(t, str == Get(complicatedJSON, "LeftOut").String())
-}
-
 func testvalid(t *testing.T, json string, expect bool) {
 	t.Helper()
 	_, ok := validpayload([]byte(json), 0)
@@ -1519,6 +1514,12 @@ func TestModifier(t *testing.T) {
 	if res != json {
 		t.Fatalf("expected '%v', got '%v'", json, res)
 	}
+	if res := Get(res, "@this").String(); res != json {
+		t.Fatalf("expected '%v', got '%v'", json, res)
+	}
+	if res := Get(res, "other.@this").String(); res != `{"hello":"world"}` {
+		t.Fatalf("expected '%v', got '%v'", json, res)
+	}
 	res = Get(res, "@pretty|@reverse|arr|@reverse|2").String()
 	if res != "4" {
 		t.Fatalf("expected '%v', got '%v'", "4", res)
@@ -2040,3 +2041,162 @@ func TestChainedModifierStringArgs(t *testing.T) {
 	res := Get("[]", `@push:"2"|@push:"3"|@push:{"a":"b","c":["e","f"]}|@push:true|@push:10.23`)
 	assert(t, res.String() == `["2","3",{"a":"b","c":["e","f"]},true,10.23]`)
 }
+
+func TestFlatten(t *testing.T) {
+	json := `[1,[2],[3,4],[5,[6,[7]]],{"hi":"there"},8,[9]]`
+	assert(t, Get(json, "@flatten").String() == `[1,2,3,4,5,[6,[7]],{"hi":"there"},8,9]`)
+	assert(t, Get(json, `@flatten:{"deep":true}`).String() == `[1,2,3,4,5,6,7,{"hi":"there"},8,9]`)
+	assert(t, Get(`{"9999":1234}`, "@flatten").String() == `{"9999":1234}`)
+}
+
+func TestJoin(t *testing.T) {
+	assert(t, Get(`[{},{}]`, "@join").String() == `{}`)
+	assert(t, Get(`[{"a":1},{"b":2}]`, "@join").String() == `{"a":1,"b":2}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2}]`, "@join").String() == `{"a":1,"b":2}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, "@join").String() == `{"a":1,"b":2,"c":3}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, `@join:{"preserve":true}`).String() == `{"a":1,"b":1,"b":2,"c":3}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, `@join:{"preserve":true}.b`).String() == `1`)
+	assert(t, Get(`{"9999":1234}`, "@join").String() == `{"9999":1234}`)
+}
+
+func TestValid(t *testing.T) {
+	assert(t, Get("[{}", "@valid").Exists() == false)
+	assert(t, Get("[{}]", "@valid").Exists() == true)
+}
+
+// https://github.com/tidwall/gjson/issues/152
+func TestJoin152(t *testing.T) {
+	var json = `{
+		"distance": 1374.0,
+		"validFrom": "2005-11-14",
+		"historical": {
+		  "type": "Day",
+		  "name": "last25Hours",
+		  "summary": {
+			"units": {
+			  "temperature": "C",
+			  "wind": "m/s",
+			  "snow": "cm",
+			  "precipitation": "mm"
+			},
+			"days": [
+			  {
+				"time": "2020-02-08",
+				"hours": [
+				  {
+					"temperature": {
+					  "min": -2.0,
+					  "max": -1.6,
+					  "value": -1.6
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 92.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T16:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.7,
+					  "max": -1.3,
+					  "value": -1.3
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 92.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T17:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.3,
+					  "max": -0.9,
+					  "value": -1.2
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 91.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T18:00:00+01:00"
+				  }
+				]
+			  },
+			  {
+				"time": "2020-02-09",
+				"hours": [
+				  {
+					"temperature": {
+					  "min": -1.7,
+					  "max": -0.9,
+					  "value": -1.5
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 91.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-09T00:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.5,
+					  "max": 0.9,
+					  "value": 0.2
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 67.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-09T01:00:00+01:00"
+				  }
+				]
+			  }
+			]
+		  }
+		}
+	  }`
+
+	res := Get(json, "historical.summary.days.#.hours|@flatten|#.humidity.value")
+	assert(t, res.Raw == `[92.0,92.0,91.0,91.0,67.0]`)
+}
+
+func TestIssue192(t *testing.T) {
+	assert(t, squash(`"000"hello`) == `"000"`)
+	assert(t, squash(`"000"`) == `"000"`)
+	assert(t, squash(`"000`) == `"000`)
+	assert(t, squash(`"`) == `"`)
+
+	assert(t, squash(`[000]hello`) == `[000]`)
+	assert(t, squash(`[000]`) == `[000]`)
+	assert(t, squash(`[000`) == `[000`)
+	assert(t, squash(`[`) == `[`)
+	assert(t, squash(`]`) == `]`)
+
+	testJSON := `0.#[[{}]].@valid:"000`
+	Get(testJSON, testJSON)
+}
+
+func TestIssue195(t *testing.T) {
+	testJSON := `\************************************` +
+		`**********{**",**,,**,**,**,**,"",**,**,**,**,**,**,**,**,**,**]`
+	Get(testJSON, testJSON)
+}
